*############## 1. GENERAL NOTES ON USING GTAP DATA ######################
$ontext

     WE RECOMMEND THE FOLLOWING SETTINGS WITH GAMS STUDIO
     1. Use GAMS Default Configuration to
        a. generate a reference file
        b, abort after a number of errors, e.g., cerr=5 or 10
     2. Run with GDZ generation, i.e., F10 
    
 1.  The GTAP database is CONSISTENT, i.e, each income to each agent
     is matched with an expenditure by an agent, and therefore the row
     and column totals roughly equate as required for GAEMPACK
     - single precision - but not at double precision.
     
 2.  While the GTAP database is consistent, it is INCOMPLETE, i.e.,
     there are missing transactions, e.g., current account transactions
     other than imports and exports of goods and services.
        This requires that 'some' transactions for every region must
        have been distorted to make the database CONSISTENT.

 3.  IDEALLY, the GTAP sets data could be used to reduce the amount of
     data read in from Excel, BUT GTAP sets data have oft times used the
     SAME labels for some regions and sectors. This can cause ordering
     problems in GAMS so it is less error prone to define many of the
     sets and maps in Excel.
       It has been indicated that GTAP may change its labels at some point
       in the future

 4.  Data v 11 are available in GDX format and therefore readable by GAMS.

 5.  The transaction values (TV) in GTAP are reported in millions of US
     dollars to SIX decimal places, i.e., GTAP reports TVs with an
     IMPLIED accuracy of a single US dollar. BUT this is not a
     correct interpretation.

 6.  In the GTAP database ALL TVs in ALL 'active' sub matrices, i.e.,
     sub matrices for which the GTAP model has behavioural relationships
     MUST have NON ZERO values. This is (apparently) because
       a.  the GTAP CGE model is implemented in GEMPACK,
       b.  the database is estimated using the FIT programme that uses
           GEMPACK, and
       c.  GEMPACK is single precision and uses log differences.

     This has some non trivial implications for users of the data
       a.  the database has a large number of datapoints that are
           NON ZERO, but do not represent real economic transactions,
           i.e., transactions that should be zero ('non zero zeros')
       b.  it is not, apparently, possible to determine which NON ZERO
           transactions are in fact 'non zero zeros' (I gave up asking
           after 10 years of getting no answer and learning that FIT
           was a GEMPACK programme),
       c.  given the presence of 'non zero zeros' it is reasonable to
           conclude that some apparent TVs should be ignored from an
           economic perspective, while from a software, e.g., GAMS,
           perspective these very small TVs may be an 'inconvenience',
       d.  it is suggested that any transactions after, at least, the
           fourth decimal place should be treated as ZEROs,
       e.  any truncating of the GTAP database risks producing
           'illogical' transactions, e.g., taxes levied on ZERO TVS,
       f.  in the GTAP database as a SAM the row and column totals are
           not identical, which restricts the ability to use accounting
           identities to check the database without adjustments,
       g.  as the number of sectors, factor and regions in the GTAP
           database increases so the data problems presented by
           'non zero zeros' increase,
       h.  GTAP v 11 has set some TVs to zero - these mostly seem to be
           tax TVs,
       i.  The GTAP 11 database contains a very large number of 'small'
           transactions.
           There are 7,619,480 datapoints in active sub matrices of the
           GTAP SAM, of which
               5,802,148 are transactions of greater than $(USD)10,
               5,097,190 are transactions of greater than $(USD)100,
               4,212,135 are transactions of greater than $(USD)1,000
               3,185,034,are transactions of greater than $(USD)10,000,
               2,140,516 are transactions of greater than $(USD)100,000
               1,228,786 are transactions of greater than $(USD)1,000,000
       j.  the definition of a 'small' transaction is not the same for the
           USA as for, say, Mauritius,
       k.  the scale of transactions varies enormously, e.g., in GTAP 11
           for 2017 the largest transaction for the region with the
           smallest 'largest' transaction, 'Rest of the World' (xtw),
           is ???, while the largest transaction for the region
           with the largest 'largest' transaction, 'USA' (usa),
           is ??? m

  7.  The routines in this programme seek simply to replicate the GTAP
      data - so there is no truncating of data, no augmentation of
      the GTAP database, no other adjustments, e.g., the REGIONAL
      HOUSEHOLD is retained.

  8.  The routines in this programme include checks at each stage of
      the process, hence there are a very large number of checks.
      This reflects the experience that for each release of the GTAP
      database it has been necessary to identify release specific
      features of the GTAP database, e.g., changes in labels used.

  9.  The routines is this programme report a lot of information about
      the 'structure' of the GTAP database; this is important because
      it allows us to better understand the transaction relationships.

  10. USERS OF THE GTAP DATABASE MUST MAKE DECISIONS ABOUT HOW THEY
      CHOOSE TO ADDRESS THE IMPLICATIONS OF THESE ASPECTS OF THE
      DATABASE IF THEY WISH TO USE THE GTAP DATABASE. SOME CHOICES
      RELATE TO THE DATA AND SOME TO THE MODEL AND SOFTWARE.

         ############ OUR DECISIONS RE THE DATA ##############

   1.  Use the SAM representation of the GTAP database produced by
       the GTAPAgg routine; this SAM uses the system devised for GTAP
       Technical Paper 22 (the original version is available at
       www.gtap.agecon.purdue.edu/resources/res_display.asp?RecordID=1645).
       A revised and updated version is available at
       http://cgemod.org.uk/tech_pap.html and is included in the
       GAMS library) - GTAP refused to publish an updated version.
   2.  The option to use accounting identities to check the database
       is too valuable to discard (we also use these identities in our
       models).
   3.  'Small' TVs that create small shares are 'problemmatic' when
       using functional forms that are standard in CGE models (CES, CET,
       LES, translog, etc), irrespective of the preferred solution
       method. Thus
       a). some truncation of the non zero elements in the GTAP SAM is
           justified, because by definition there are large numbers of
           'non zero zeros', BUT it is not possible to determine which
           'non zero zeros' represent non transactions.
       b). the degree of truncation should be user defined.
   5.  It is more efficient to resolve all data issues once; hence
       we choose to convert all GTAP data from HAR to GDX format once
       and at the same time truncate the non zero elements and FORCE
       the resultant SAM to satisfy the requisite accounting identities.
   6.  The imposition of the requisite accounting identities is achieved
       manually; the choice to 'balance' the SAM manually, rather than
       by, say, RAS, was largely taken to avoid creating new
       'small' transactions, but also to avoid proportionately large
       changes in tax and/or subsidy rates.
   7.  The trade margin accounts do not 'balance' and are 'tricky' to
       resolve using only within region data; hence we create a dummy
       region, 'GLOBE', that supplies and demands all margin services.
       This region is used in our models and is implicit in the GTAP
       model.
   8.  The whole of the GTAP database is transformed into a
       transactions SAM and a series of linked satellite accounts using
       a common set of account labels.
   
$offtext
